import React from "react";
import { AxisLeft, AxisBottom } from '@vx/axis';
import { AxisRight, AxisTop } from '@vx/axis';
import { AreaStack } from '@vx/shape';
import { scaleLinear, scaleOrdinal } from '@vx/scale';
import { Group } from '@vx/group';
import { extent } from 'd3-array';
import toy_data from '../higgs_mass_dist.json';

console.log(toy_data)

const  trans_data = [
	{left: 0, right: 50, a: 10, b: 20, c: 5, d: 0},
	{left: 50, right: 60, a: 20, b: 43, c: 9, d: 3},
	{left: 60, right: 150, a: 10, b: 40, c: 5, d: 2},
	{left: 150, right: 200, a: 2, b: 4, c: 3, d: 0},
]

const keys = ["a", "b", "d", "c"]
// accessors
const getX = (d) => {
	return (d.left + d.right) / 2
}



// Child iterates over stacksitems = a list of rects
// rects given freedom to highlight single bins or whole processes
const Histogram = ({
  uhepp,
  yScale,
  xScale,
  colorScale,
  children
}) => {

  const n_bins = uhepp.bins.edges.length - 1
  let objects = []

  for (let bin_i=0; bin_i < n_bins; bin_i++) {
    uhepp.stacks.forEach((stack, stack_index) => {
      let bottom = 0
      stack.content.forEach((stack_item, si_i) => {
        const y_value = stack_item["yield"].map(name => uhepp.yields[name].base[bin_i]).reduce((a, b) => a + b)
        if (stack.type == "stepfilled") {
          objects.push(<rect
            key={`rect-${stack_index}-${si_i}-${bin_i}`}
            x={xScale(uhepp.bins.edges[bin_i])}
            width={xScale(uhepp.bins.edges[bin_i + 1]) - xScale(uhepp.bins.edges[bin_i])}
            y={yScale(bottom + y_value)}
            height={yScale(bottom) - yScale(bottom + y_value)}
            fill={stack_item.style ? stack_item.style.color : '#1f77b4'}
            />)
        } else if (stack.type == "points") {
          const width = xScale(uhepp.bins.edges[bin_i + 1]) - xScale(uhepp.bins.edges[bin_i])
          const stat2 = stack_item["yield"].map(name => uhepp.yields[name].stat[bin_i]).map(a => a*a).reduce((a, b) => a + b)
          const stat = Math.sqrt(stat2)
          const stat_up = xScale(y_value + stat) - xScale(y_value)
          const stat_down = xScale(y_value) - xScale(y_value - stat)
          const gx = (xScale(uhepp.bins.edges[bin_i]) + xScale(uhepp.bins.edges[bin_i + 1])) / 2
          const gy = yScale(bottom + y_value)
          objects.push(<g transform={`translate(${gx}, ${gy})`}
            key={`p-${stack_index}-${si_i}-${bin_i}`}
            >
            <circle
              cx="0"
              cy="0"
              stroke="#000"
              r="3"/>
            <line
              y1="0"
              y2="0"
              stroke="#000"
              stroke-width="2"
              x1={-width / 2}
              x2={+width / 2} />
            <line
              y1={-stat_down}
              y2={stat_up}
              stroke-width="2"
              stroke="#000"
              x1="0"
              x2="0" />
            </g>)
        }
        bottom += y_value


      })
    })
  }

  return objects
}


const UheppHist = ({width, height}) => {
	const margin = {
		top: 60,
		bottom: 60,
		left: 80,
		right: 80,
	}

	const xMax = width - margin.left - margin.right
	const yMax = height - margin.top - margin.bottom

	const xScale = scaleLinear({
		range: [0, xMax],
		domain: extent(toy_data.bins.edges),
	})
	const yScale = scaleLinear({
		range: [yMax, 0],
		domain: [0, 80],
	})

	const purple1 = '#6c5efb';
	const purple2 = '#c998ff';
	const purple3 = '#ffcc00';
	const purple4 = '#a44afe';
	const colorScale = scaleOrdinal({
		domain: keys,
		range: [purple1, purple2, purple3, purple4],
	});

	return (
  <>
    <style>
     svg rect, svg line {'{shape-rendering: crispEdges }'}
     </style>
			<svg width={width} height={height}>
				<Group top={margin.top} left={margin.left}>

         <Histogram
            uhepp={toy_data}
            stack_index={0}
            xScale={xScale}
            yScale={yScale}
          />
  
		 		<AxisBottom
						scale={xScale}
						top={yMax}
						label={toy_data.variable.symbol}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
						tickTransform="translate(0, -8)"
					/>
					<AxisLeft
						scale={yScale}
						top={0}
						left={0}
						label={'Events'}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
						tickTransform="translate(8, 0)"
					/>
					<AxisRight
						scale={yScale}
						top={0}
						left={xMax}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(-8, 0)"
					/>
					<AxisTop
						scale={xScale}
						top={0}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(0, 8)"
					/>
				</Group>
			</svg>
  </>
	)
}

export default function Toy() {
	const width = 750
	const height = 500

	return <UheppHist width={width} height={height} />
}


